/*
 * Модель LoRaWan базовой станции

  Список тригеров, которые нужно использовать
  
    connect( bool ) Соединение с сервером 
      @param Bool статус соединения
      
    auth - Статус аутентификации


 */
 var LORA_CONVERTER = Backbone.Model.extend({
  
  // Токен авторизации
  token: "",
  
  // Соединение
  socket: null,
  
  // Идентификатор БС
  gatawayId : "",
  
  
  defaults: {
    id: 0,
    about: "",
    ip: "",
    port: 0,
    type: 0,
    login: "",
    pass: ""
  },
  
  url: function () {
    return "/api/lora/" + this.get('id') + "/";
  },


  // Обрабатываем только те данные, которые подходят под формат
  parse : function( data ){
    if( data instanceof Object ){
      return data
    }
  },


  // Запрос данных на БС
  get_devices_to :function( collect , callback ){
    
    var callback = callback

    collect.reset()

    this.fetch({
      url : this.url() + "meters/", 
      success : function( _ , data ){
        collect.add(data)
        if( callback instanceof Function){
          callback()
        }
      }
    })

  },

  /*
    0 (CONNECTING)  The connection is not yet open.
    1 (OPEN)        The connection is open and ready to communicate.
    2 (CLOSING)     The connection is in the process of closing.
    3 (CLOSED)      The connection is closed or couldn't be opened
  */
  connect : function(){
    this.socket = new WebSocket("ws://" + this.get('ip') + ":" + this.get('port') )
    if( this.socket.readyState >= 2 ){
      this.trigger('connect' , false )
      return
    }

    var self = this;
    
    this.socket.addEventListener("open" , function(){
      self.trigger('connect' , this.readyState == 1 )
    })
  },

  
  // Авторизация на БС
  // После отправки запроса начинаем слушать БС на атворизацию
  auth: function () {
    if( !( this.socket instanceof WebSocket ) ){
      return
    }
    this.socket.send( this.__auth() );

    var self = this;

    this.socket.addEventListener("message" , function( event ){

      try{
        var data = JSON.parse( event.data )
        if( data.cmd == "auth_resp" ){
          self.trigger("auth" ,  data.status && data.token != "" )
          self.token = data.token;
        }
      }catch(e){
        console.log(e)
      }

    })
  },


  // Получение всех устройств на БС
  get_devices: function () {
    if( !( this.socket instanceof WebSocket ) ){
      return
    }
    return this.socket.send(JSON.stringify({
      cmd   : "get_devices_req",
      token : this.token
    }));
  },
  
  // Получение всех БС
  get_gataways: function () {
    if( !( this.socket instanceof WebSocket ) ){
      return
    }
    this.socket.send(JSON.stringify({
      cmd   : "get_gateways_req",
      token : this.token
    }));
  },  


  save_gataways: function ( device ) {
    if( !( this.socket instanceof WebSocket ) ){
      return
    }
    this.socket.send(JSON.stringify({
      cmd   : "manage_gateways_req",
      gateway_list : [ device.toJSON() ]
    }));
  },
  
  drop_gataway: function ( device ) {
    if( !( this.socket instanceof WebSocket ) ){
      return
    }
    this.socket.send(JSON.stringify({
      cmd   : "delete_gateways_req",
      gateway_list : [ device.get('gatewayId') ]
    }));
  },
  
  
  /*
   * 
   * Получение всех приборов на БС
   */
  get_all_devices: function () {
    if( !( this.socket instanceof WebSocket ) ){
      return
    }
    this.socket.send( this.__all_devices() );
  },
  


  /*
   * 
   * Получение всех приборов на БС
   */
  get_device_appdata: function () {
    if( !( this.socket instanceof WebSocket ) ){
      return
    }
    this.socket.send( this.__device_appdata_req() );
  },
  
  
  /*
   * Удаление прибора
   * 
   * @param {Backbone.Model}
   */
  delete_device: function ( model ) {
    if( typeof model == "undefined" ){
      return false
    }
    this.socket.send( this.__delete_device( model ) );
    return true
  },
  
  /*
   * Сохранение типа прибора
   * 
   * @param {Backbone.Model}
   */
   save_type_device: function (model) {
    this.socket.send( this.__save_type_device( model ) );
  },
  

  /*
   * Сохранение прибора
   * 
   * @param {Backbone.Model}
   */
   save_counter: function (model) {
    this.socket.send( this.__update_device( model ) );
  },

  add_counter: function (model) {
    this.socket.send( this.__add_device( model ) );
  },
  
  
  /*
   * Генерация команды для добавления устройства
   * 
   * @param {Backbone.Model}
   * @return {String}
   */
   __add_device : function( model ){
    console.log("add")
    return JSON.stringify({
      cmd           : "manage_devices_req",
      devices_list  : [ model.__toJSON() ],
      token         : this.token
    });
  },


  // Команда для авторизации
  // @return String
  __auth : function(){
    return JSON.stringify({
      cmd   : "auth_req",
      login : this.get('login'),
      password : this.get('pass')
    })
  },
  
  
  /*
   * Генерация команды для редактирования устройства
   * @param {Backbone.Model}
   * @return {String}
   */
   __update_device : function( model ){

    var changed = model.changed

    changed.devEui = model.get('devEui')

    return JSON.stringify({
      cmd : "manage_devices_req",
      devices_list: [changed],
      token: this.token
    });
  },
  
 /*
  22414136 - >>>
  21824326 <<-- !!!
  014D0346
 */

  /*
   * Генерация команды для редактирования типа устройства
   * @param {Backbone.Model}
   * @return {String}
   */
   __save_type_device : function( model ){
    return JSON.stringify({
      cmd: "manage_device_appdata_req",
      data_list: [{
        devEui      : model.get('devEui'),
        device_type : model.get('device_type')
      }],
      token : this.token
    });
  },
  
  
  /*
   * Генерация команды для удаления устройства
   * @param {Backbone.Model}
   * @return {String}
   */
   __delete_device : function( model ){
    return JSON.stringify({
      cmd : "delete_devices_req",
      devices_list: [ model.get('devEui') ],
      token: this.token
    });
  },
  
  
  /*
   * Генерация команды для получения всех устройств на БС
   * 
   * @return {String}
   */
   __all_devices : function(){
    return JSON.stringify({
      cmd   : "get_device_appdata_req",
      token : this.token
    });
  },


  /*
    Генерация команды для получения прибор-тип с БС
  */
  __device_appdata_req : function(){
    return JSON.stringify({
      cmd   : "get_device_appdata_req",
      token : this.token
    });
  },
  
  
  /*
    Валидация модели
    @return Object | undefined
    */
  validate : function(){

    // Проверка валидности адреса
    if( typeof this.is_valid_ip() != "undefined" ){
      return this.is_valid_ip();
    }


    // Проверка видности порта
    if( typeof this.is_valid_port() != "undefined" ){
      return this.is_valid_port()
    }
  },

  /*
    Проверяет валиден ли адрес
    @return String | undefined
    */
    is_valid_ip : function(){
      if( this.get('ip') == "" ){
        return "empty ip"
      }

      var rg = new RegExp(/^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/);

      if( !rg.test( this.get('ip') ) ){
        return "invalid ip"
      }
    },


  /*
    Проверяет валиден ли порт
    @return String | undefined
    */
    is_valid_port : function(){

      if( isNaN( this.get('port') ) ){
        return "port is not a string"
      }

      if( this.get('port') <= 0 || this.get('port') > 65535 ){
        return "invalid port"
      }
    },


    /*
  
      Синхронизация лоровского прибора к нам на сервер

      @param {Model}      {LORA_DEVICE}  Какой прибор мы хотим сохранить
      @param {Collection} {MeterTypes}   Типы приборов на сервере нагрузки
      @param {Collection} {MeterTypes}   

      @trigger sync_to_db( new_model ) Отработает, когда будет создана модель

    */
    sync_to_db : function( device , meter_types , _lora_types ){

      if( typeof ( device || meter_types ) == "undefined" ){
        return
      }

      // Первое что надо сделать это создать модель

      if( typeof METER_MODEL == "undefined" ){
        return
      }


      var metertype = meter_types.findWhere({ devEui : device.get('devEui') });
      var type = _lora_types.findWhere({ id : Number( metertype.get('device_type') ) })

      var new_model = new METER_MODEL({
        converter_id : this.get('id'),
        num_serial : device.get('devEui'),
        about : device.get('devName')
      });

      var self = this;
      var device = device;

      new_model.save({ type_string : type.get('name') } , {
        success : function( ){
          device.set('sync' , false)
        }
      })

    },

    // Удаление из БД прибра которого нет на сервере у лора
    rm_to_db : function( _meter ){
      var self = this;
      _meter.destroy({ success : function(_meter){
        _meter.set('sync' , false)
        self.trigger('rm_to_db' , _meter )
      }})
    }

  });